@@ -20,20 +20,36 @@
  * Authors: Rodrigo Moya <rodrigo.moya@canonical.com>
  */
 
+#include <config.h>
 #include <string.h>
+#include <glib/gi18n-lib.h>
 #include "e-book-backend-couchdb.h"
+#include <libedataserver/eds-version.h>
 #include <libedata-book/e-book-backend-sexp.h>
 #include <libedata-book/e-data-book.h>
 #include <libedata-book/e-data-book-view.h>
 #include <dbus/dbus-glib.h>
-#include <gnome-keyring.h>
 
-#define COUCHDB_REVISION_PROP                "X-COUCHDB-REVISION"
 #define COUCHDB_UUID_PROP                    "X-COUCHDB-UUID"
 #define COUCHDB_APPLICATION_ANNOTATIONS_PROP "X-COUCHDB-APPLICATION-ANNOTATIONS"
 
 G_DEFINE_TYPE (EBookBackendCouchDB, e_book_backend_couchdb, E_TYPE_BOOK_BACKEND);
 
+#if EDS_CHECK_VERSION(2, 31, 0)
+#define ERROR_QUARK e_book_backend_couchdb_error_quark ()
+
+static GQuark
+e_book_backend_couchdb_error_quark (void)
+{
+	static GQuark error;
+
+        if (!error)
+                error = g_quark_from_static_string ("evolution_couchdb");
+
+        return error;
+}
+#endif
+
 static void
 get_current_time (gchar time_string[100])
 {
@@ -47,7 +63,7 @@ get_current_time (gchar time_string[100])
 }
 
 static EContact *
-contact_from_couch_document (CouchdbDocument *document)
+contact_from_couch_document (EBookBackendCouchDB *couchdb_backend, CouchdbDocument *document)
 {
 	EContact *contact;
 	char *str;
@@ -79,12 +95,13 @@ contact_from_couch_document (CouchdbDocument *document)
 		}
 	}
 
+	/* Add entry to the revisions table */
+	g_hash_table_insert (couchdb_backend->revisions,
+			     g_strdup (couchdb_document_get_id (document)),
+			     g_strdup (couchdb_document_get_revision (document)));
+
 	/* Fill in the EContact with the data from the CouchDBDocument */
 	contact = e_contact_new ();
-	e_vcard_add_attribute_with_value (E_VCARD (contact),
-					  e_vcard_attribute_new (NULL, COUCHDB_REVISION_PROP),
-					  couchdb_document_get_revision (document));
-
 	e_contact_set (contact, E_CONTACT_UID, (const gpointer) couchdb_document_get_id (document));
 
 	contact_name.family = (char *) couchdb_document_contact_get_last_name (COUCHDB_DOCUMENT_CONTACT (document));
@@ -716,12 +733,13 @@ couch_document_from_contact (EBookBackendCouchDB *couchdb_backend, EContact *con
 	document = couchdb_document_contact_new ();
 
 	str = e_contact_get_const (contact, E_CONTACT_UID);
-	if (str)
+	if (str != NULL) {
 		couchdb_document_set_id (COUCHDB_DOCUMENT (document), str);
 
-	str = e_vcard_attribute_get_value (e_vcard_get_attribute (E_VCARD (contact), COUCHDB_REVISION_PROP));
-	if (str)
-		couchdb_document_set_revision (COUCHDB_DOCUMENT (document), str);
+		str = g_hash_table_lookup (couchdb_backend->revisions, str);
+		if (str != NULL)
+			couchdb_document_set_revision (COUCHDB_DOCUMENT (document), str);
+	}
 
 	contact_name = (EContactName *) e_contact_get (contact, E_CONTACT_NAME);
 	if (contact_name != NULL) {
@@ -927,14 +945,18 @@ document_updated_cb (CouchdbDatabase *database, CouchdbDocument *document, gpoin
 	EContact *contact;
 	EBookBackendCouchDB *couchdb_backend = E_BOOK_BACKEND_COUCHDB (user_data);
 
-	contact = contact_from_couch_document (document);
+	contact = contact_from_couch_document (couchdb_backend, document);
 	if (!contact)
 		return;
 
-	e_book_backend_notify_update (E_BOOK_BACKEND (couchdb_backend), contact);
-
 	/* Add the contact to the cache */
 	e_book_backend_cache_add_contact (couchdb_backend->cache, contact);
+	g_hash_table_insert (couchdb_backend->revisions,
+			     g_strdup (couchdb_document_get_id (document)),
+			     g_strdup (couchdb_document_get_revision (document)));
+
+	/* Notify listeners */
+	e_book_backend_notify_update (E_BOOK_BACKEND (couchdb_backend), contact);
 
 	g_object_unref (G_OBJECT (contact));
 }
@@ -944,24 +966,39 @@ document_deleted_cb (CouchdbDatabase *database, const char *docid, gpointer user
 {
 	EBookBackendCouchDB *couchdb_backend = E_BOOK_BACKEND_COUCHDB (user_data);
 
-	e_book_backend_notify_remove (E_BOOK_BACKEND (couchdb_backend), docid);
-
 	/* Remove the contact from the cache */
 	e_book_backend_cache_remove_contact (couchdb_backend->cache, docid);
+	g_hash_table_remove (couchdb_backend->revisions, docid);
+
+	/* Notify listeners */
+	e_book_backend_notify_remove (E_BOOK_BACKEND (couchdb_backend), docid);
 }
 
+#if EDS_CHECK_VERSION(2, 31, 0)
+static void
+e_book_backend_couchdb_load_source (EBookBackend *backend,
+				    ESource *source,
+				    gboolean only_if_exists,
+				    GError **ret_error)
+#else
 static GNOME_Evolution_Addressbook_CallStatus
 e_book_backend_couchdb_load_source (EBookBackend *backend,
 				    ESource *source,
 				    gboolean only_if_exists)
+#endif
 {
 	gchar *uri;
 	const gchar *property;
 	GError *error = NULL;
 	GSList *doc_list, *sl;
+	const gchar *db_name;
 	EBookBackendCouchDB *couchdb_backend = E_BOOK_BACKEND_COUCHDB (backend);
 
+#if EDS_CHECK_VERSION(2, 31, 0)
+	g_return_if_fail (E_IS_BOOK_BACKEND_COUCHDB (couchdb_backend));
+#else
 	g_return_val_if_fail (E_IS_BOOK_BACKEND_COUCHDB (couchdb_backend), GNOME_Evolution_Addressbook_OtherError);
+#endif
 
 	if (couchdb_backend->couchdb != NULL)
 		g_object_unref (G_OBJECT (couchdb_backend->couchdb));
@@ -969,6 +1006,10 @@ e_book_backend_couchdb_load_source (EBookBackend *backend,
 		g_object_unref (G_OBJECT (couchdb_backend->database));
 	if (couchdb_backend->cache != NULL)
 		g_object_unref (G_OBJECT (couchdb_backend->cache));
+	if (couchdb_backend->revisions != NULL) {
+		/* Remove all IDs<->revisions pairs */
+		g_hash_table_remove_all (couchdb_backend->revisions);
+	}
 
 	/* create CouchDB main object */
 	couchdb_backend->using_desktopcouch = FALSE;
@@ -976,8 +1017,13 @@ e_book_backend_couchdb_load_source (EBookBackend *backend,
 	property = e_source_get_property (source, "couchdb_instance");
 	if (g_strcmp0 (property, "user") == 0) {
 		if (! (couchdb_backend->couchdb = COUCHDB_SESSION (desktopcouch_session_new ()))) {
+#if EDS_CHECK_VERSION(2, 31, 0)
+			g_set_error (ret_error, ERROR_QUARK, 100, _("Could not create DesktopcouchSession object"));
+			return;
+#else
 			g_warning ("Could not create DesktopcouchSession object");
 			return GNOME_Evolution_Addressbook_NoSuchBook;
+#endif
 		}
 
 		couchdb_backend->using_desktopcouch = TRUE;
@@ -989,39 +1035,69 @@ e_book_backend_couchdb_load_source (EBookBackend *backend,
 
 		if (! (couchdb_backend->couchdb = couchdb_session_new (uri))) {
 			g_free (uri);
+#if EDS_CHECK_VERSION(2, 31, 0)
+			g_set_error (ret_error, ERROR_QUARK, 100, _("Could not create CouchdbSession object"));
+			return;
+#else
 			return GNOME_Evolution_Addressbook_NoSuchBook;
+#endif
 		}
 
 		g_free (uri);
 	}
 
+	db_name = e_source_get_property (source, "couchdb_database");
+	if (db_name == NULL)
+		db_name = "contacts";
+
 	/* check if only_if_exists */
 	error = NULL;
 	couchdb_backend->database = couchdb_session_get_database (couchdb_backend->couchdb,
-								  "contacts",
+								  db_name,
 								  &error);
 	if (couchdb_backend->database == NULL) {
-		if (error) {
+		if (error)
 			g_warning ("Could not get CouchDB database info: %s", error->message);
-			g_error_free (error);
+
+		if (only_if_exists) {
+#if EDS_CHECK_VERSION(2, 31, 0)
+			if (error) {
+				g_set_error (ret_error, ERROR_QUARK, error->code, "%s: %s",
+					     _("Could not get CouchDB database info"), error->message);
+				g_error_free (error);
+			} else
+				g_set_error (ret_error, ERROR_QUARK, 0, _("Could not open database %s"), db_name);
+
+			return;
+#else
+			return GNOME_Evolution_Addressbook_NoSuchBook;
+#endif
 		}
 
-		if (only_if_exists)
-			return GNOME_Evolution_Addressbook_NoSuchBook;
+		if (error)
+			g_error_free (error);
 		
 		/* if it does not exist, create it */
 		error = NULL;
 		if (!couchdb_session_create_database (couchdb_backend->couchdb,
-						      "contacts",
+						      db_name,
 						      &error)) {
+#if EDS_CHECK_VERSION(2, 31, 0)
+			g_set_error (ret_error, ERROR_QUARK, error->code, "%s: %s",
+				     _("Could not create 'contacts' database"), error->message);
+#endif
 			g_warning ("Could not create 'contacts' database: %s", error->message);
 			g_error_free (error);
 
+#if EDS_CHECK_VERSION(2, 31, 0)
+			return;
+#else
 			return GNOME_Evolution_Addressbook_PermissionDenied;
+#endif
 		}
 
 		couchdb_backend->database = couchdb_session_get_database (couchdb_backend->couchdb,
-									  "contacts",
+									  db_name,
 									  &error);
 	}
 
@@ -1039,7 +1115,7 @@ e_book_backend_couchdb_load_source (EBookBackend *backend,
 		EContact *contact;
 		CouchdbDocument *document = COUCHDB_DOCUMENT (sl->data);
 
-		contact = contact_from_couch_document (document);
+		contact = contact_from_couch_document (couchdb_backend, document);
 		if (contact != NULL) {
 			e_book_backend_cache_add_contact (couchdb_backend->cache, contact);
 			g_object_unref (G_OBJECT (contact));
@@ -1060,7 +1136,9 @@ e_book_backend_couchdb_load_source (EBookBackend *backend,
 	e_book_backend_set_is_loaded (backend, TRUE);
 	e_book_backend_set_is_writable (backend, TRUE);
 
+#if !EDS_CHECK_VERSION(2, 31, 0)
 	return GNOME_Evolution_Addressbook_Success;
+#endif
 }
 
 static void
@@ -1074,9 +1152,15 @@ e_book_backend_couchdb_remove (EBookBackend *backend, EDataBook *book, guint32 o
 		couchdb_backend->cache = NULL;
 	}
 
+	g_hash_table_remove_all (couchdb_backend->revisions);
+
 	/* We don't remove data from CouchDB, since it would affect other apps,
 	   so just report success */
+#if EDS_CHECK_VERSION(2, 31, 0)
+	e_data_book_respond_remove (book, opid, NULL);
+#else
 	e_data_book_respond_remove (book, opid, GNOME_Evolution_Addressbook_Success);
+#endif
 }
 
 static char *
@@ -1086,15 +1170,15 @@ e_book_backend_couchdb_get_static_capabilities (EBookBackend *backend)
 }
 
 static EContact *
-put_document (EBookBackendCouchDB *couchdb_backend, CouchdbDocument *document)
+put_document (EBookBackendCouchDB *couchdb_backend, CouchdbDocument *document, GError **error)
 {
-	GError *error = NULL;
+	*error = NULL;
 
-	if (couchdb_database_put_document (couchdb_backend->database, document, &error)) {
+	if (couchdb_database_put_document (couchdb_backend->database, document, error)) {
 		EContact *new_contact;
 
 		/* couchdb_document_put sets the ID for new documents, so need to send that back */
-		new_contact = contact_from_couch_document (document);
+		new_contact = contact_from_couch_document (couchdb_backend, document);
 
 		/* Add the new contact to the cache */
 		e_book_backend_cache_add_contact (couchdb_backend->cache, new_contact);
@@ -1102,8 +1186,7 @@ put_document (EBookBackendCouchDB *couchdb_backend, CouchdbDocument *document)
 		return new_contact;
 	} else {
 		if (error != NULL) {
-			g_warning ("Could not PUT document: %s\n", error->message);
-			g_error_free (error);
+			g_warning ("Could not PUT document: %s\n", ((GError *) error)->message);
 		}
 	}
 
@@ -1118,27 +1201,27 @@ e_book_backend_couchdb_create_contact (EBookBackend *backend,
 {
 	EContact *contact, *new_contact;
 	CouchdbDocument *document;
+	GError *error = NULL;
 	EBookBackendCouchDB *couchdb_backend = E_BOOK_BACKEND_COUCHDB (backend);
 
 	contact = e_contact_new_from_vcard (vcard);
-	if (!contact) {
-		e_data_book_respond_create (book, opid, GNOME_Evolution_Addressbook_OtherError, NULL);
-		return;
-	}
-
 	document = couch_document_from_contact (couchdb_backend, contact);
-	if (!document) {
-		e_data_book_respond_create (book, opid, GNOME_Evolution_Addressbook_OtherError, NULL);
-		g_object_unref (G_OBJECT (contact));
-		return;
-	}
 
 	/* save the contact into the DB */
-	if ((new_contact = put_document (couchdb_backend, document)) != NULL) {
+	if ((new_contact = put_document (couchdb_backend, document, &error)) != NULL) {
+#if EDS_CHECK_VERSION(2, 31, 0)
+		e_data_book_respond_create (book, opid, NULL, new_contact);
+#else
 		e_data_book_respond_create (book, opid, GNOME_Evolution_Addressbook_Success, new_contact);
+#endif
 		g_object_unref (new_contact);
-	} else
+	} else {
+#if EDS_CHECK_VERSION(2, 31, 0)
+		e_data_book_respond_create (book, opid, error, NULL);
+#else
 		e_data_book_respond_create (book, opid, GNOME_Evolution_Addressbook_OtherError, NULL);
+#endif
+	}
 
 	/* free memory */
 	g_object_unref (G_OBJECT (contact));
@@ -1152,11 +1235,11 @@ e_book_backend_couchdb_remove_contacts (EBookBackend *backend,
 					GList *id_list)
 {
 	GList *l, *deleted_ids = NULL;
+	GError *error = NULL;
 	EBookBackendCouchDB *couchdb_backend = E_BOOK_BACKEND_COUCHDB (backend);
 
 	for (l = id_list; l != NULL; l = l->next) {
 		CouchdbDocument *document;
-		GError *error = NULL;
 		const gchar *uid = (const gchar *) l->data;
 
 		document = couchdb_database_get_document (couchdb_backend->database, uid, &error);
@@ -1188,11 +1271,12 @@ e_book_backend_couchdb_remove_contacts (EBookBackend *backend,
 					deleted_ids = g_list_append (deleted_ids, (gpointer) uid);
 					e_book_backend_cache_remove_contact (couchdb_backend->cache, uid);
 				} else {
-					if (error != NULL) {
+					if (error != NULL)
 						g_debug ("Error deleting document: %s", error->message);
-						g_error_free (error);
-					} else
+					else
 						g_debug ("Error deleting document");
+
+					break;
 				}
 
 				/* Free memory */
@@ -1203,30 +1287,44 @@ e_book_backend_couchdb_remove_contacts (EBookBackend *backend,
 				if (couchdb_database_delete_document (couchdb_backend->database, document, &error)) {
 					deleted_ids = g_list_append (deleted_ids, (gpointer) uid);
 					e_book_backend_cache_remove_contact (couchdb_backend->cache, uid);
+					g_hash_table_remove (couchdb_backend->revisions, uid);
 				} else {
-					if (error != NULL) {
+					if (error != NULL)
 						g_debug ("Error deleting document: %s", error->message);
-						g_error_free (error);
-					} else
+					else
 						g_debug ("Error deleting document");
+
+					break;
 				}
 			}
 		} else {
-			if (error != NULL) {
+			if (error != NULL)
 				g_debug ("Error getting document: %s", error->message);
-				g_error_free (error);
-			} else
+			else
 				g_debug ("Error getting document");
+
+			break;
 		}
 	}
 
 	if (deleted_ids) {
+#if EDS_CHECK_VERSION(2, 31, 0)
+		e_data_book_respond_remove_contacts (book, opid, NULL, deleted_ids);
+#else
 		e_data_book_respond_remove_contacts (book, opid,
 						     GNOME_Evolution_Addressbook_Success, deleted_ids);
+#endif
 		g_list_free (deleted_ids);
-	} else
+	} else {
+#if EDS_CHECK_VERSION(2, 31, 0)
+		if (error == NULL)
+			error = g_error_new_literal (ERROR_QUARK, 100, _("Could not remove contacts from CouchDB"));
+		e_data_book_respond_remove_contacts (book, opid, error, NULL);
+#else
 		e_data_book_respond_remove_contacts (book, opid,
 						     GNOME_Evolution_Addressbook_OtherError, NULL);
+#endif
+	}
 }
 
 static void
@@ -1237,27 +1335,27 @@ e_book_backend_couchdb_modify_contact (EBookBackend *backend,
 {
 	EContact *contact, *new_contact;
 	CouchdbDocument *document;
+	GError *error = NULL;
 	EBookBackendCouchDB *couchdb_backend = E_BOOK_BACKEND_COUCHDB (backend);
 
 	contact = e_contact_new_from_vcard (vcard);
-	if (!contact) {
-		e_data_book_respond_modify (book, opid, GNOME_Evolution_Addressbook_OtherError, NULL);
-		return;
-	}
-
 	document = couch_document_from_contact (couchdb_backend, contact);
-	if (!document) {
-		e_data_book_respond_modify (book, opid, GNOME_Evolution_Addressbook_OtherError, NULL);
-		g_object_unref (G_OBJECT (document));
-		return;
-	}
 
 	/* save the contact into the DB */
-	if ((new_contact = put_document (couchdb_backend, document)) != NULL) {
+	if ((new_contact = put_document (couchdb_backend, document, &error)) != NULL) {
+#if EDS_CHECK_VERSION(2, 31, 0)
+		e_data_book_respond_modify (book, opid, NULL, new_contact);
+#else
 		e_data_book_respond_modify (book, opid, GNOME_Evolution_Addressbook_Success, new_contact);
+#endif
 		g_object_unref (new_contact);
-	} else
+	} else {
+#if EDS_CHECK_VERSION(2, 31, 0)
+		e_data_book_respond_modify (book, opid, error, NULL);
+#else
 		e_data_book_respond_modify (book, opid, GNOME_Evolution_Addressbook_OtherError, NULL);
+#endif
+	}
 
 	/* free memory */
 	g_object_unref (G_OBJECT (contact));
@@ -1279,16 +1377,27 @@ e_book_backend_couchdb_get_contact (EBookBackend *backend,
 
 		g_object_unref (G_OBJECT (contact));
 		if (vcard != NULL) {
+#if EDS_CHECK_VERSION(2, 31, 0)
+			e_data_book_respond_get_contact (book, opid, NULL, vcard);
+#else
 			e_data_book_respond_get_contact (book,
 							 opid,
 							 GNOME_Evolution_Addressbook_Success,
 							 vcard);
+#endif
 			g_free (vcard);
 			return;
 		}
-	}
+	} else {
+#if EDS_CHECK_VERSION(2, 31, 0)
+		GError *error;
 
-	e_data_book_respond_get_contact (book, opid, GNOME_Evolution_Addressbook_ContactNotFound, "");
+		error = g_error_new (ERROR_QUARK, 100, _("Contact %s could not be found in CouchDB contacts database"), id);
+		e_data_book_respond_get_contact (book, opid, error, NULL);
+#else
+		e_data_book_respond_get_contact (book, opid, GNOME_Evolution_Addressbook_ContactNotFound, NULL);
+#endif
+	}
 }
 
 static void
@@ -1313,7 +1422,11 @@ e_book_backend_couchdb_get_contact_list (EBookBackend *backend,
 		g_object_unref (G_OBJECT (contact));
 	}
 
+#if EDS_CHECK_VERSION(2, 31, 0)
+	e_data_book_respond_get_contact_list (book, opid, NULL, contacts);
+#else
 	e_data_book_respond_get_contact_list (book, opid, GNOME_Evolution_Addressbook_Success, contacts);
+#endif
 }
 
 static void
@@ -1342,7 +1455,11 @@ e_book_backend_couchdb_start_book_view (EBookBackend *backend,
 		g_object_unref (G_OBJECT (contact));
 	}
 
+#if EDS_CHECK_VERSION(2, 31, 0)
+	e_data_book_view_notify_complete (book_view, NULL);
+#else
 	e_data_book_view_notify_complete (book_view, GNOME_Evolution_Addressbook_Success);
+#endif
 }
 
 static void
@@ -1381,8 +1498,12 @@ e_book_backend_couchdb_get_required_fields (EBookBackend *backend,
 	field_name = e_contact_field_name (E_CONTACT_GIVEN_NAME);
 	fields = g_list_append (fields, g_strdup (field_name));
 
+#if EDS_CHECK_VERSION(2, 31, 0)
+	e_data_book_respond_get_required_fields(book, opid, NULL, fields);
+#else
 	e_data_book_respond_get_required_fields(book, opid,
 						GNOME_Evolution_Addressbook_Success, fields);
+#endif
 
 	g_list_foreach (fields, (GFunc) g_free, NULL);
 	g_list_free (fields);
@@ -1459,8 +1580,12 @@ e_book_backend_couchdb_get_supported_fields (EBookBackend *backend,
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_IM_SKYPE)));
 	fields = g_list_append (fields, g_strdup (e_contact_field_name (E_CONTACT_IM_YAHOO)));
 
+#if EDS_CHECK_VERSION(2, 31, 0)
+	e_data_book_respond_get_supported_fields (book, opid, NULL, fields);
+#else
 	e_data_book_respond_get_supported_fields (book, opid,
 						  GNOME_Evolution_Addressbook_Success, fields);
+#endif
 
 	g_list_foreach (fields, (GFunc) g_free, NULL);
 	g_list_free (fields);
@@ -1473,21 +1598,36 @@ e_book_backend_couchdb_get_supported_auth_methods (EBookBackend *backend, EDataB
 
 	auth_methods = g_list_append (auth_methods, g_strdup ("plain/password"));
 
+#if EDS_CHECK_VERSION(2, 31, 0)
+	e_data_book_respond_get_supported_auth_methods (book, opid, NULL, auth_methods);
+#else
 	e_data_book_respond_get_supported_auth_methods (book, opid,
 							GNOME_Evolution_Addressbook_Success, auth_methods);
+#endif
 
 	g_list_foreach (auth_methods, (GFunc) g_free, NULL);
 	g_list_free (auth_methods);
 }
 
+#if EDS_CHECK_VERSION(2, 31, 0)
+static void
+e_book_backend_couchdb_cancel_operation (EBookBackend *backend, EDataBook *book, GError **ret_error)
+{
+}
+#else
 static GNOME_Evolution_Addressbook_CallStatus
 e_book_backend_couchdb_cancel_operation (EBookBackend *backend, EDataBook *book)
 {
 	return GNOME_Evolution_Addressbook_CouldNotCancel;
 }
+#endif
 
 static void
+#if EDS_CHECK_VERSION(2, 31, 0)
+e_book_backend_couchdb_set_mode (EBookBackend *backend, EDataBookMode mode)
+#else
 e_book_backend_couchdb_set_mode (EBookBackend *backend, GNOME_Evolution_Addressbook_BookMode mode)
+#endif
 {
 }
 
@@ -1518,10 +1658,15 @@ e_book_backend_couchdb_dispose (GObject *object)
 		couchdb_backend->cache = NULL;
 	}
 
-	if (couchdb_backend->database) {
+	if (couchdb_backend->database != NULL) {
 		g_object_unref (G_OBJECT (couchdb_backend->database));
 		couchdb_backend->database = NULL;
 	}
+
+	if (couchdb_backend->revisions != NULL) {
+		g_hash_table_destroy (couchdb_backend->revisions);
+		couchdb_backend->revisions = NULL;
+	}
 }
 
 static void
@@ -1560,4 +1705,5 @@ e_book_backend_couchdb_init (EBookBackendCouchDB *backend)
 	backend->couchdb = NULL;
 	backend->database = NULL;
 	backend->cache = NULL;
+	backend->revisions = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 }
